/*
 * track.java
 *
 * Created on January 3, 2008, 2:26 PM
 * Copyright: 2008, Dirkjan Krijnders <dirkjan@krijnders.net>
 * 
 * track.java is part of GetWBT
 * 
 * GetWBT is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    GetWBT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GetWBT.  If not, see <http://www.gnu.org/licenses/>.


 */

package com.pythsoft;

import com.mindprod.ledatastream.LEDataOutputStream;
import java.io.IOException;
import java.util.Date;
import henson.midp.Float11;

// import java.util.
/**
 *
 * @author Administrator
 */
public class track {
    public int trackNumber = 0;
    public int offset = 0;
    private int numberOfPoints = 0;
    public int time = 0;
    public double distance = 0;
    private double lastLatitude = 0;
    private double lastLongitude = 0;
    private boolean firstPoint;
    private Date startDate;
    private Date stopDate;
    private Logger logger = null;
    
    /** Creates a new instance of track */
    public track() {
        firstPoint = true;
    }
    
    public track(int number) {
        firstPoint = true;
        trackNumber = number;
        logger = Logger.getInstance();
    }
    
    public track(int number, int offset, int numberOfPoints) {
        firstPoint = true;
        trackNumber = number;
        this.offset = offset;
        this.numberOfPoints = numberOfPoints;
    }
    
    public void writeTrack(LEDataOutputStream str) throws IOException {
        logger.message(4, "Number of points for track " + this.trackNumber + " is " + this.numberOfPoints +
                " Startdate: " + this.startDate);
        str.writeInt(trackNumber);
        str.writeInt(offset);
        str.writeInt(numberOfPoints);
        str.writeInt(time);
        str.writeDouble(distance);
    }
    
    public int numberOfPoints() {
        return this.numberOfPoints;
    }
    
    public void setNumberOfPoints(int np) {
        this.numberOfPoints = np;
        logger.message(4, "Number of points for track " + this.trackNumber + " is " + this.numberOfPoints);
    }

    public void setOffset(int offset) {
        this.offset = offset;
        logger.message(4, "Offset for track " + this.trackNumber + " is " + this.offset);
    }

    public double getDistance() {
        return distance;
    }
    
    public Date getStartDate() {
        return this.startDate;
    }
    
    public void setTime(int time) {
        this.time = time;
    }

    public void setTrackNumber(int trackNumber) {
        this.trackNumber = trackNumber;
    }
    
    public void setDistance(double distance) {
        this.distance = distance;
    }
    
    public void setStartDate(Date startDate) {
        logger.message(4, "StartDate: " + startDate);
        this.startDate = startDate;
        setStopDate(startDate); // Make sure a stopdate is set...
    }
    
    public void setStopDate(Date stopDate) {
        logger.message(4, "Stopdate: " + stopDate);
        this.time = (int) ((stopDate.getTime()/1000) - (startDate.getTime()/1000));
        logger.message(4, "Diff in seconds is " + this.time);
        this.stopDate = stopDate;
    }
    
    public void addPoint(double latitude, double longitude) {
        latitude = Math.toRadians(latitude); // / 180 * 3.1415;
        longitude = Math.toRadians(longitude); // / 180 * 3.1415;
        if (!firstPoint) {
              distance += calcDistance(latitude, longitude, lastLatitude, lastLongitude);
        } else {
            firstPoint = false;
//            logger.message(4, "First point: " + latitude + "," + longitude );
        }
        numberOfPoints++;
        lastLatitude = latitude;
        lastLongitude = longitude;
    }
    
    private double calcDistance(double latitude, double longitude, double lastLatitude, double lastLongitude) {
        double R = 6371d;
        double dLat = lastLatitude - latitude;
        double dLon = lastLongitude - longitude;
            
        double a = Math.sin(dLat/2) * Math.sin(dLat/2) +
                          Math.cos(latitude) * Math.cos(lastLatitude) *
                          Math.sin(dLon/2) * Math.sin(dLon/2);
        return R * 2 * Float11.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    }
    
}
