/*
 * WBTController.java
 *
 * Created on July 9, 2007, 8:42 AM
 * Copyright: 2007, 2008, Dirkjan Krijnders <dirkjan@krijnders.net>
 * 
 * WBTController.java is part of GetWBT
 * 
 * GetWBT is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    GetWBT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GetWBT.  If not, see <http://www.gnu.org/licenses/>.


 */

package com.pythsoft;

import com.substanceofcode.tracker.model.Settings;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import javax.bluetooth.*;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.midlet.*;
import javax.microedition.rms.RecordStoreException;
import com.mindprod.ledatastream.LEDataOutputStream;
import javax.microedition.lcdui.*;
import java.util.Vector;

/**
 *
 * @author Administrator
 */
public class WBTController {
    WBT201 gpsDevice = null;
    InputStreamReader isr = null;
    OutputStreamWriter osr = null;
    char[] data = null;
    Settings sc = null;
    MIDlet md = null;
    private Logger logger = null;
    /** Creates a new instance of WBTController */
    public WBTController(MIDlet amd) {
        md = amd;
        logger = Logger.getInstance();
//        initialize();
    };
    
    public void initialize() {
        String addr = "";
        try {
            sc = Settings.getInstance(md);
            //sc.load();
            
//            sc.setStringProperty("btaddr","000b0d87147f");
            addr = sc.getStringProperty("btaddr", "");
        } catch (IOException e) {
            e.printStackTrace();
        } catch (RecordStoreException rse) {
            rse.printStackTrace();
        }
        if (addr == "")
        {
            addr = "000b0d86fdca";
        }
        logger.message(4, "btaddr: "+addr);
        if ( isr == null || osr == null) {
            try {
                StreamConnection btconnection = BTConnect(addr);
                isr = new InputStreamReader(btconnection.openInputStream());
                osr = new OutputStreamWriter(btconnection.openOutputStream());
            } catch(IOException e) {
                System.out.println(e.toString());
            }
        }
        if (gpsDevice == null)
            gpsDevice = new WBT201(isr, osr);
        
        try {
            gpsDevice.login();
            gpsDevice.login();
        } catch (IOException e) {
            System.out.println("Login failed");
        }
        logger.message(4, "Connection made");
    }
    
    private StreamConnection BTConnect(String addr) throws IOException {
         String url = "btspp://" + addr + ":1";
                return (StreamConnection) Connector.open(url);
    }
    
    public String GpsInfo() {
        String info = null;
        try {
            info = gpsDevice.getName();
        } catch(IOException e) {
            info = e.toString();
        }
        return info;
    }
        
    public boolean readAndWrite(LEDataOutputStream fos, org.netbeans.microedition.lcdui.WaitScreen screen) {
        boolean status = true;
        try {
            gpsDevice.readAndWrite(fos, screen);
        } catch (IOException e) {
            System.out.println(e.toString());
            status = false;
        }    
        return status;
    }
    
    public void erase() {
        try {
            gpsDevice.erase();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public int getDataLength() {
        int size = 0;
        if (data == null || data.length == 0)
            try {
                size = gpsDevice.bytesToBeRead();
            } catch (IOException e) {
                System.out.println("bla");
            }
        else
            size = data.length;
        return size;
    }
    
    public char[] getData() {
        return data;
    }
    
    public int getCapacity()
    {
        int size = 0;
        try {
            size = gpsDevice.getCapacity();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return size;
    }
    
    public float getPercentage()
    {
        int length = this.getDataLength();
        int capacity = this.getCapacity();
        logger.message(4, length + " / " + capacity + " = " + ((float)length/(float)capacity)) ;
        float percentage = 100f * (float)length / (float)capacity;
        return percentage;
    }
    
    public void close() {
        try {
            gpsDevice.logout();
        } catch (IOException e) {
            System.out.println("IOException during logout");
        }
    }
    
    public int getConfigDynamicPlatform(){
        try {
            return gpsDevice.getConfigDynamicPlatform();
        } catch (IOException e) {
            System.out.println("IOException during logout");
        }
        return 1;
    }
    
    public Double getHwVersion() {
        try {
            Double version = gpsDevice.getHwVersion();
            logger.message(4, "HW Version:" + version);
            return version;
        } catch (IOException e) {
            System.out.println("IOException during hardware version");
        }
        return new Double(0.0);    
    }    
    
    public Double getSwVersion() {
        try {
            return gpsDevice.getSwVersion();
        } catch (IOException e) {
            System.out.println("IOException during software version");
        }
        return new Double(0.0);
    }
    
    public int getBytesToRead(){
        try {
            return gpsDevice.bytesToBeRead();
        } catch (IOException e) {
            System.out.println("IOException during bytes to read");
        }
        return 0;
    }
    
    public String getSerialNumber(){
        try {
            return gpsDevice.getSerialNumber();
        } catch (IOException e) {
            System.out.println("IOException during getSerialNumber");
        }
        return new String("");
    }
        
    public int getTrackCount(){
        return gpsDevice.getTrackCount();
    }
    
    public Vector tracks() {
        return gpsDevice.getTracks();
    }
    
    public int getPointCount() {
        try {
            return gpsDevice.getPointCount();
        } catch (IOException e) {
            System.out.println("IOException during getSerialNumber");
        }
        return 0;
    }
}